# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from mock import patch

from charmworld.jobs.ingest import ProofIngestJob
from charmworld.testing import factory
from charmworld.testing import JobTestBase


class ProofTest(JobTestBase):

    def test_charm_proof_library_not_found(self):
        # If the charm-proof library cannot be found, an error is logged
        # and the proof loop aborted.
        nonexistant_path = '/does/not/exist/'
        job = ProofIngestJob()
        handler = self.get_handler('charm.proof')
        job.setup(proof_path=nonexistant_path)
        log_messages = [record.getMessage() for record in handler.buffer]
        err = ("proof error before processing began: "
               "could not find charm proof path.")
        self.assertIn(err, log_messages)
        self.assertIn('CHARM_PROOF_PATH: %s' % nonexistant_path, log_messages)

    def test_no_proof_for_deleted_branch(self):
        # If the branch of a charm is deleted from Launchpad, the proof
        # job does nothing.
        job = ProofIngestJob()
        job.setup()
        charm = factory.get_charm_json(branch_deleted=True)
        with patch.object(job, 'proofer') as mock:
            job.run(charm)
        self.assertFalse(mock.called)
