# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime
from pyramid.exceptions import NotFound

from charmworld.models import find_charms
from charmworld.models import is_reviewed


# View helpers used throughout various views

def format_change(change, charm=None):
    """Format a bzr commit."""
    parts = []
    date = datetime.fromtimestamp(change["created"])
    parts.append(unicode(date.strftime("%Y/%m/%d")))
    parts.append(name_filter(change['committer']))
    parts.append(change["message"][:100])
    if charm is not None:
        parts.append(
            u'(<a href="'
            'https://bazaar.launchpad.net/%s/revision/%s">revno %s</a>)' % (
                charm.branch_spec, change["revno"], change["revno"]))
    return u" ".join(parts)


def format_proof(charm):
    proof = charm.proof
    messages = []
    for e in proof.get('c', []):
        messages.append('crit: %s' % e)
    for e in proof.get('e', []):
        messages.append("err: %s" % e)
    for e in proof.get('w', []):
        messages.append('warn: %s' % e)
    return messages


def found(res):
    if res is None:
        raise NotFound()
    return res


def interfaces(db):
    interfaces = {}

    # This is all static between loads
    results = find_charms(
        db, {}, {"name": 1, "i_provides": 1, "i_requires": 1,
                 'label': 1, 'short_url': 1, 'owner': 1, 'store_url': True})

    for i in results:
        for p in i["i_provides"]:
            if not p in interfaces:
                interfaces[p] = i_info = {
                    "name": p, "provided_by": [], "required_by": []}
            else:
                i_info = interfaces[p]
            i_info["provided_by"].append(i)

        for d in i["i_requires"]:
            if not d in interfaces:
                interfaces[d] = i_info = {
                    "name": d, "provided_by": [], "required_by": []}
            else:
                i_info = interfaces[d]
            i_info["required_by"].append(i)

    # sort them
    interfaces = interfaces.values()
    interfaces.sort(lambda x, y: cmp(x["name"], y["name"]))

    for i in interfaces:
        i["provided_by"] = unique_sort(i["provided_by"])
        i["required_by"] = unique_sort(i["required_by"])

    return interfaces


def name_filter(n):
    if isinstance(n, list):
        n = n[0]
    idx = n.find('<')
    if idx == -1:
        return n
    return n.split("<", 1)[0].strip()


def result_sorter(a, b):
    review_cmp = cmp(is_reviewed(b), is_reviewed(a))
    if review_cmp != 0:
        return review_cmp

    from charmworld.views import SERIES_SORT
    series = cmp(SERIES_SORT.index(a['series']),
                 SERIES_SORT.index(b['series']))
    if series != 0:
        return series

    return cmp(b['relevance'], a['relevance'])


def sub_filter(d, req):
    if req.GET.get('sub'):
        d['subordinate'] = True
        return d, True
    return d, False


def unique_sort(charms):
    names = []
    results = []
    for c in charms:
        if c["name"] in names:
            if not is_reviewed(c):
                continue
            else:
                idx = names.index(c['name'])
                results.pop(idx)
                names.pop(idx)
        results.append(c)
        names.append(c["name"])
    results.sort(lambda x, y: cmp(x["name"], y["name"]))
    return results
