# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import pymongo
from pyramid.view import view_config

from charmworld import utils
from charmworld.models import Charm
from charmworld.views.helpers import format_proof
from charmworld.views.helpers import sub_filter


# 3m
REPORT_CACHE = 180


@view_config(
    route_name="tools-core-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def core_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.core_review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def charm_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-store-missing",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/store-missing.pt")
def charm_store_missing(request):
    query = {"store_data.errors": {"$exists": True}}
    queue = [Charm(data) for data in request.db.charms.find(query)]
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-proof-errors",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/charm-proof-errors.pt")
def proof_errors(request):
    query, sub_only = sub_filter(
        {"proof.e": {"$exists": True}, "promulgated": True}, request)
    sort = [("series", pymongo.DESCENDING), ("name", pymongo.ASCENDING)]
    charms = [Charm(data) for data in request.db.charms.find(query, sort=sort)]

    return {
        'charms': charms,
        'format_proof': format_proof,
        'sub_only': sub_only,
    }
