# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
"""
Update the qa questions collection used by the qa form.

Removes the providers questions from the reliable section because that
information is automated.
"""

new_category_data = [
    (
        u'reliable',
        u'Reliable',
        [
            (u'Check for integrity from upstream source', 1, u''),
            (u'Fail gracefully if upstream source goes missing', 1, u''),
            (u'Contain a suite of tests with the charm that pass', 1, u''),
            (u'Passes tests from Jenkins on jujucharms.com', 1, u''),
        ]
    ),
]


def upgrade(db, index_client):
    """Update each category of questions from new_category_data.

    This update elaborates on the 001_* migration. Subsequent qa form
    mirgrations can replace new_category_data and run this same method.
    """
    for cat in new_category_data:
        name = cat[0]
        category_dict = {
            'name': name,
            'description': cat[1],
            'questions': [{
                'id': '{0}_{1}'.format(cat[0].lower(), i),
                'description': q[0],
                'points': q[1],
                'extended_description': q[2]
            } for i, q in enumerate(cat[2])]
        }
        db.qa.update({'name': name}, category_dict)
